import google.auth
from google.oauth2 import service_account
from googleapiclient.discovery import build


class BaseGoogleClient:
    def __init__(self, key_file=None):
        if key_file is None:
            self.credential = self.create_credential_default()
        else:
            self.credential = self.create_credential_from_file(key_file)
        self.service = self.create_service()

    def create_credential_from_file(self, key_file):
        credential = service_account.Credentials.from_service_account_file(key_file, scopes=self.scope)

        return credential

    def create_credential_default(self):
        credential, project = google.auth.default(scopes=self.scope)

        return credential

    def create_service(self):

        service = build(self.service_name, self.service_version, credentials=self.credential)

        return service



from marshmallow import Schema, ValidationError, fields, validate

from ._googleclient import BaseGoogleClient


class SearchConsole(BaseGoogleClient):
    """Google SearchConsol query

    인스턴스의 기본설정 값(GOOGLE_APPLICATION_CREDENTIALS)이나 service account 키파일을 사용하여 search console 서비스 객체를 생성하고
    사용자 사이트(site_url)의 검색 트래픽에 대한 통계 정보를 얻을 수 있다.

    https://developers.google.com/webmaster-tools/search-console-api-original/v3/searchanalytics/query

    사용 예시) ``https://soomgo.com`` 에 대해 2021-11-30 ~ 2021-11-30까지 image 검색에 대한 통계정보를 받아온다.
    구글 API 서비스생성시 키파일을 사용하거나 인스턴스의 기본설정을 사용한다.

    .. code-block:: python

        >>> from soomgogather.google import SearchConsole

        >>> site_url = "https://soomgo.com"

        >>> service = SearchConsole('path/service_account_key.json')
        >>> # when using the default setting
        >>> # service = SearchConsole()

        >>> params = {
        ...     'start_dt': '2021-11-01',
        ...     'end_dt': '2021-11-30',
        ...     'type': 'image'
        ... }

        >>> r = service.query(site_url, params=params)

        >>> if r['rows']:
        ...     print(r['rows'])
    """

    scope = ['https://www.googleapis.com/auth/webmasters.readonly']
    service_name = 'searchconsole'
    service_version = 'v1'

    class _SearchConsoleSchema(Schema):
        start_dt = fields.Str(attribute='startDate', required=True)
        end_dt = fields.Str(attribute='endDate', required=True)
        dimensions = fields.List(
            fields.Str(),
            attribute='dimensions',
            validate=validate.ContainsOnly(['country', 'device', 'date', 'page', 'query', 'searchAppearance']),
        )
        row_limit = fields.Int(attribute='rowLimit', validate=validate.Range(max=25000))
        start_row = fields.Int(attribute='startRow', validate=validate.Range(min=0))
        type = fields.Str(
            attribute='type', validate=validate.OneOf(['web', 'video', 'image', 'news', 'googleNews', 'discover'])
        )
        dimension_filter_groups = fields.List(fields.Str(), attribute='dimensionFilterGroups')
        data_state = fields.Str(attribute='dataState', validate=validate.OneOf(['all', 'final']))
        aggregation_type = fields.Str(attribute='aggregationType')

    def _get_params(self, params):

        if "data_state" in params:
            params["data_state"] = params["data_state"].lower()

        try:
            return self._SearchConsoleSchema().load(params)
        except ValidationError as err:
            raise ValueError(f"incorrect parameters: {err}")

    def query(self, site_url, params={}):
        """전달한 필터와 조건에 맞는 검색 트래픽에 대한 정보를 반환한다.

        **params:**
         - start_dt: 통계를 추출할 기간의 시작일, YYYY-MM-DD PT (UTC - 7:00/8:00) (required)
         - end_dt: 통계를 추출할 기간의 종료일, YYYY-MM-DD PT (UTC - 7:00/8:00) (required)
         - dimensions: 결과를 그룹핑할 키, 지정안하거나 여러개를 지정가능, 결과셋의 keys값
            - country, device, date, page, query, searchAppearance
         - row_limit: 반환되는 결과의 row수 지정
            - default 1000, maximum = 25000
         - start_row: 반환되는 결과의 시작 row 번호
            - 결과셋은 0부터 시작하는 인덱스
            - 양수만 가능
         - type: 구글의 검색타입 필터
            - web(default), video, image, news, discover, googleNews(구글 뉴스 앱)
         - dimension_filter_groups: dimensions 로 그룹핑시 필터 및 조건
            - groupType: 현재는 'and' 조건만 지원
            - filters: dimension의 필터 설정 리스트
                - 형식: (dimension name) (an operator) (a value)
                - dimension: 필터를 지정할 dimension 이름 (country, device, date, query, searchAppearance)
                - operator: contains, equals, notEquals, includingRegex, excludingRegex
                - expression: 필터 값
         - data_state: all(fresh data 포함) 또는 final 값으로 지정 (대소문자 구분안함)
         - aggregation_type: 결과의 aggregation(집합) 타입, auto(default), byPage(by URI), byProperty(by property)
        """
        response = self.service.searchanalytics().query(siteUrl=site_url, body=self._get_params(params)).execute()
        return response



from marshmallow import Schema, ValidationError, fields, validate

from ._searchad import BaseSearchAD


class RelKwdStat(BaseSearchAD):
    """Naver SearchAd API RelKwdstat

    Naver SearchAd에서 API키를 발급받은 api_key, secret_key, customer_id를 사용하여 RelKwdStat 클래스 객체를 생성한다.
    생성한 RelKwdStat 객체를 사용하여 연관 검색어의 통계정보(Query count, Click count, CTR, 경쟁력 지표)에 대한 정보를 얻을 수 있다.

    http://naver.github.io/searchad-apidoc/#/tags/RelKwdStat

    사용 예시) 숨고와 숨고의 연관검색어에 대한 통계정보를 받아온다.

    .. code-block:: python

        >>> from soomgogather.naver import RelKwdStat

        >>> rel_keyword_stat = Relkwdstat(api_key='_', secret_key='_', customer_id='_')

        >>> r = rel_keyword_stat.list(params={
        ...    'hint_keywords' : '숨고,soomgo',
        ... })

        >>> if r.status_code == 200:
        ...     print(r.json())
    """

    class _RelkwdstatSchema(Schema):
        # 아래 다섯개중에 하나는 파라미터로 들어가야함
        site_id = fields.Str(attribute='siteId')
        biztp_id = fields.Int(attribute='biztpId')
        hint_keywords = fields.Str(attribute='hintKeywords')
        event = fields.Int(attribute='event')
        month = fields.Int(attribute='month')

        # 설정하지 않아도 defautl=0, _get_param 에서 입력받은 파라미터의 수를 확인하기 위해 명시적으로 0으로 설정
        show_detail = fields.Int(attribute='showDetail', validate=validate.OneOf([0, 1]))

    def _get_params(self, params):
        try:
            return self._RelkwdstatSchema().load(params)
        except ValidationError as err:
            raise ValueError(f"incorrect parameters: {err}")

    def list(self, params={}):
        """파라미터의 조건에 맞는 키워드의 지표를 반환한다.

        **params:**

        ``show_detail`` 을 제외한 5개의 파라미터 중 1개 이상 지정되어야 결과값이 나옴
         - site_id: 채널타입이 SITE인 비즈니스 채널 ID(nccBusinessChannelId)
         - biztp_id: 비즈니스 타입 ID
         - hint_keywords: 검색어 - comma(,)로 구분하여 5개까지 가능, 공백 허용 안됨
                ex) ```soomgo, 숨고``` (X) ```soomgo,숨 고``` (X) ```soomgo,숨고``` (O)
         - event: 시즌테마
            - https://gist.github.com/naver-searchad/235202ffb08f9433b6f7cb10e45875f7#file-seasonal_event_code-md
         - month: 월
         - show_detail: 상세정보 조회 여부
            - 0: 상세정보 조회안함(기본값)
            - 1: 상세정보 조회
        """
        return self.call('GET', '/keywordstool', params=self._get_params(params))



# Copyright 2021 Brave Mobile Co., Ltd.
import setuptools

install_requires = [
    "marshmallow",
    "google-api-core",
    "google-api-python-client",
]

tests_requires = [
    "black>=20.8b1",
    "flake8>=3.8.2",
    "pytest",
    "pytest-cov",
    "tox",
    "Sphinx>=3.2.1",
    "docutils==0.15.2",
    "isort",
    "requests_mock",
    "pytest-mock",
] + install_requires


def get_long_description() -> str:
    return (
        open("README.md", encoding="utf8").read().strip() + "\n\n" + open("CHANGELOG.md", encoding="utf8").read().strip()
    )


setuptools.setup(
    name="Soomgo-gather",
    version=open("VERSION").read().strip(),
    url="https://github.com/Soomgo-Platform/soomgo-gather",
    description="",
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    keywords="Naver,NaverSearchAD,GA,collect,data,gather,soomgo-gather,soomgo",
    author="Brave Mobile Co., Ltd.",
    author_email="platform@soomgo.com",
    project_urls={
        "Changelog": "https://github.com/Soomgo-Platform/soomgo-gather/blob/main/CHANGELOG.md",
        "Tracker": "https://github.com/Soomgo-Platform/soomgo-gather/issues",
    },
    license="MIT",
    packages=setuptools.find_packages(where="src"),
    package_dir={"": "src"},
    python_requires=">=3.6",
    install_requires=[],
    extras_require={"test": tests_requires},
    zip_safe=False,
    # test_suite=''
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.8",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)



from marshmallow import Schema, ValidationError, fields

from ._searchad import BaseSearchAD


class Bizmoney(BaseSearchAD):
    """Naver SearchAd API Bizmoney

    Naver SearchAd에서 API키를 발급받은 api_key, secret_key, customer_id를 사용하여 Bizmoney 클래스 객체를 생성한다.
    생성한 Bizmoney 객체로 Bizmoney의 잔액, 사용금액, 충전내역, 잠금상태에 대한 정보를 얻을 수 있다.

    http://naver.github.io/searchad-apidoc/#/tags/Bizmoney

    사용 예시) 2021-11-18 에 사용한 금액을 받아온다.

    .. code-block:: python

        >>> from soomgogather.naver import Bizmoney

        >>> bizmoney = Bizmoney(api_key='_', secret_key='_', customer_id='_')

        >>> r = bizmoney.exhaust(params={
        ...    'search_start_dt': '20211118',
        ...    'search_end_dt': '20211118',
        ... })

        >>> if r.status_code == 200:
        ...     print(r.json())
    """

    class _BizmoneySchema(Schema):
        search_start_dt = fields.Str(attribute='searchStartDt', required=True)
        search_end_dt = fields.Str(attribute='searchEndDt', required=True)
        stat_dt = fields.Str(attribute='statDt')

    def _get_params(self, params):
        try:
            return self._BizmoneySchema().load(params)
        except ValidationError as err:
            raise ValueError(f"incorrect parameters: {err}")

    def status(self):
        """Bizmoney 잔액과 환불/예산의 잠금상태를 반환한다."""
        return self.call('GET', '/billing/bizmoney')

    def cost(self, params={}):
        """파라미터로 전달한 기간의 Bizmoney 사용된 금액을 반환한다.

        **params:**
         - search_start_dt: 조회 시작일 (required)
         - search_end_dt: 조회 종료일 (required)
        """
        return self.call('GET', '/billing/bizmoney/cost', params=self._get_params(params))

    def charge(self, params={}):
        """파라미터로 전달한 기간의 Bizmoney 충전 내역을 반환한다.

        **params:**
         - search_start_dt: 조회 시작일 (required)
         - search_end_dt: 조회 종료일 (required)
        """
        return self.call('GET', '/billing/bizmoney/histories/charge', params=self._get_params(params))

    def exhaust(self, params={}):
        """파라미터로 전달한 기간의 Bizmoney 공제된 내역을 반환한다.

        **params:**
         - search_start_dt: 조회 시작일 (required)
         - search_end_dt: 조회 종료일 (required)
        """
        return self.call('GET', '/billing/bizmoney/histories/exhaust', params=self._get_params(params))

    def period(self, params={}):
        """파라미터로 전달한 기간의 일자별 BizMoney 상태값을 반환한다.

        **params:**
         - search_start_dt: 조회 시작일 (required)
         - search_end_dt: 조회 종료일 (required)
        """
        return self.call('GET', '/billing/bizmoney/histories/period', params=self._get_params(params))



import base64
import hashlib
import hmac
import time

import requests
from marshmallow import Schema, fields


class BaseSearchAD:
    domain = 'https://api.naver.com'

    allowed_method = ['GET', 'POST', 'DELETE', 'PUT']

    encode = 'utf-8'

    class HeaderSchema(Schema):
        content_type = fields.Str(data_key='Content-Type', dump_default='application/json; charset=UTF-8')
        timestamp = fields.Str(data_key='X-Timestamp')
        api_key = fields.Str(data_key='X-API-KEY')
        customer = fields.Str(data_key='X-Customer')
        signature = fields.Str(data_key='X-Signature')

    def __init__(self, api_key, secret_key, customer_id):
        self.api_key = api_key
        self.secret_key = secret_key
        self.customer_id = customer_id
        self.timestamp = str(round(time.time() * 1000))

    def _generate_signature(self, method, path):
        _hash = hmac.new(
            bytes(self.secret_key, self.encode),
            bytes(f'{self.timestamp}.{method.upper()}.{path}', self.encode),
            hashlib.sha256,
        )
        _hash.hexdigest()
        return base64.b64encode(_hash.digest())

    def make_header(self, method, path):
        return self.HeaderSchema().dump(
            {
                'timestamp': self.timestamp,
                'api_key': self.api_key,
                'signature': self._generate_signature(method, path),
                'customer': self.customer_id,
            }
        )

    def call(self, method, path, params={}):
        return getattr(requests, method.lower())(
            self.domain + path, params=params, headers=self.make_header(method, path)
        )
